 

 
import java.util.*;

/**
 * Esta classe cria BiBliotecas Multimdia capazes
 * de gerir Items Multimdia tais como CDs, DVDs, etc.
 * 
 * Biblioteca = Nome + HashMap(ItemMultimedia)
 * 
 * @author PPIV
 * @version 2.0 (Maio 2006)
 */
public class BibMultimedia {

    // Variveis de Instncia
    
    private String nome;     // Nome da Biblioteca 
    private HashMap items;   // HashMap de Items
    

    
    
    
    /**
     * Construtor da Biblioteca Multimdia
     */
    public BibMultimedia(String nome) {
        this.nome = nome; 
        this.items = new HashMap();
    }
    
    /**
     * Construtor completo da Biblioteca Multimdia.
     * Este construtor aceita um hashmap de Items como
     * parmetro e faz uma "deep copy" para inicializar 
     * a varivel de instncia items, por forma a no 
     * existir partilha de objectos entre o HashMap que
     * foi dado como parmetro e o HashMap interno.
     * 
     * Note-se que por forma a trabalhar orientado ao conceito,
     *  passado como parmetro um Map - o que significa que se podem 
     * enviar como parmetro HashMap, HashTable, TreeMap, etc.
     * 
     */
    public BibMultimedia(String nome, Map itemsInic) {
        this.nome = nome;
        ItemMultimedia item;
        
        this.items = new HashMap();
        for(Iterator it = itemsInic.values().iterator(); it.hasNext(); ) {
           item = (ItemMultimedia) it.next();
           this.items.put(item.getTitle(), item.clone());
       } 
    }


    
    /**
     * Construtor de cpia
     */
   
    public BibMultimedia(BibMultimedia bib) {
      HashMap itemsInic;    
      this.nome = bib.getName();
      itemsInic = bib.getItems();
      this.items = new HashMap();
      
      ItemMultimedia item;
      for(Iterator it = itemsInic.values().iterator(); it.hasNext(); ) {
        item = (ItemMultimedia) it.next();
        this.items.put(item.getTitle(), item.clone());
       }
       
       // Uma alternatica possvel, se bem que no fazendo a cpia, seria a 
       // que utiliza o mtodo putAll das HashMap.
       // Teriamos assim:
       //
       // this.items.putAll(itemsInic);
       //
    }    
    
    
    // Mtodo de instncia privado - auxiliar
    
    private HashMap getItems() {
      return this.items;   
    }    
    
    
    // Mtodos de Instncia pblicos

    public String getName() {       
      return this.nome;
    }
    
    public void setNome(String nome) {
      this.nome = nome;   
    }    
    
    /**
     * Adiciona um Item  base de dados
     *
     * /
    public void addItem(ItemMultimedia item) throws ExistingItemMultimediaException{   
        // ATENO: fazer clone() de item.
        //  sempre necessrio providenciar clone() em todas 
        // as classes !!! 
        
        if (this.items.containsKey(item.getTitle())) 
          throw new ExistingItemMultimediaException();
        else  
        this.items.put(item.getTitle(), item.clone());
    } 
    
   /**
    * Verifica se existe um Item dado o seu ttulo.
    */
   public boolean existeTitulo(String titulo) {
       return this.items.containsKey(titulo) ;
    }
   
   /**
    * Procura um Item pelo seu ttulo e devolve uma cpia do mesmo.
    * O mtodo assume que tal Item existe cf. teste anterior.
    */
   public ItemMultimedia consultaTitulo(String titulo) {
       ItemMultimedia item = (ItemMultimedia) items.get(titulo);
       return (ItemMultimedia) item.clone() ;
    }
    
  /**
   * Remove da biblioteca o Item de Titulo dado
   */
  public void removeItem(String titulo) {
      items.remove(titulo) ;
  }
   
   /**
    * Devolve uma lista contendo cpias de todos os Items que tm 
    * por artista ou realizador o nome dado como parmetro.
    */
   public Collection listaPorAutor(String autor) {
       ItemMultimedia item;
       ArrayList itemsOK = new ArrayList();
       for(Iterator it = items.values().iterator(); it.hasNext();) {
           item = (ItemMultimedia) it.next();
           if( (item.getTitle()).equals(autor) )
                itemsOK.add(item.clone());
       }
       return itemsOK;
   }
    
   /**
    * Determina o nmero total de Pistas de todos os CDs que
    * existem na biblioteca.
    */ 
   public int totalPistasCD() {
       ItemMultimedia item;
       int totalPistas = 0;
       for(Iterator it = items.values().iterator(); it.hasNext(); ) {
           item = (ItemMultimedia) it.next();
           if( item instanceof CD )
                totalPistas += ((CD) item).getNumberOfTracks();
       }
       return totalPistas;
   }
     
  /**
   * Realiza o toString() de toda a Biblioteca
   * 
   * @return String com a informao de cada componente
   */
  public String toString() {
      StringBuffer s = new StringBuffer();
      ItemMultimedia item;
      
      s.append(" Biblioteca Multimedia { ");
      s.append(" Nome: ");
      s.append(this.nome);
      s.append(" Items: ");
      s.append(this.items.toString());

      return s.toString();
   }   
   
   
   /**
    * Mtodo que implementa a igualdade entre objectos que sejam 
    * Bibliotecas Multimdia.
    * 
    * @return boolean com indicao se so ou no iguais
    */
   
   public boolean equals(Object o) {
   
       if(this == o)
         return true;
       if((o!= null) && this.getClass()==o.getClass()) {
           BibMultimedia bib = (BibMultimedia)o;
           return (this.nome.equals(bib.getName()) && this.items.equals(bib.getItems()));
        }
        else
          return false;
          
   }    
   

    /**
     * Mtodo que devolve uma cpia do objecto a que  enviado.
     * 
     * @return Object com a cpia estrutural do estado
     */

    public Object clone() {
      return new BibMultimedia(this);
    }    
                  
}
