import java.io.*;

/**
 * Esta classe contm a informao comum a todos os discos.
 * A sua constituio tem dois grandes objectivos:
 * 
 * 1 - serve como mecanismo de reutilizao, permitindo conter todo o estado
 *      e comportamento comuns.
 * 2 - providencia um tipo de dados nico, por forma a que por generalizao,
 *      todos os objectos que sejam instncias de subclasses de disco, possam ser 
 *      referenciados.
 *      Permite assim que novas subclasse de ItemMultimedia possam ser vistas como sendo deste tipo,
 *      sem grande alterao nas classes cliente (como a que modela a Biblioteca Multimdia).
 * 
 * @author Michael Kolling and David J. Barnes (alterado por anr)
 * @version Maio 2006
 */
public abstract class ItemMultimedia implements Serializable
{
    private String title;
    private int playingTime;
    private boolean gotIt;
    private String comment;
    
    /**
     * Cosntrutor Vazio 
     */
      
    
    public ItemMultimedia()
    {
        this.title = "";
        this.playingTime = 0;
        this.gotIt = false;
        this.comment = "<no comment>";
    }

    /**
     * Construtor parametrizado
     */
    public ItemMultimedia(String title, int time, boolean gotIt, String comment)
    {
        this.title = title;
        this.playingTime = time;
        this.gotIt = gotIt;
        this.comment = comment;
    }


    public ItemMultimedia(ItemMultimedia it) {
     
        this.title = it.getTitle();
        this.playingTime = it.getPlayingTime();
        this.gotIt = it.getOwn();
        this.comment = it.getComment();
        
    }    

    /**
     * Colocar um comentrio no ItemMultimedia
     */
    public void setComment(String comment)
    {
        this.comment = comment;
    }

    /**
     * Devolver o comentrio de um ItemMultimedia
     * 
     * @return String com o comentrio
     */
    public String getComment()
    {
        return this.comment;
    }

    /**
     * Mtodo que permite alterar a flag que indica se possuimos uma
     * cpia do ItemMultimedia.
     */
    public void setOwn(boolean ownIt)
    {
        this.gotIt = ownIt;
    }

    /**
     * Devolve o valor de own, indicando se possuimos ou no uma
     * cpia do ItemMultimedia
     * 
     * @return true se possuimos o ItemMultimedia, false caso contrrio
     */
    public boolean getOwn()
    {
        return this.gotIt;
    }


    /**
     * Mtodo que devolve o nome do ItemMultimedia
     * 
     * @return String com o ttulo
     */
    
    public String getTitle() {
      return this.title;
    }
    
    
    /**
     * Mtodo que devolve o tempo de execuo de um ItemMultimedia
     * 
     * @return int tempo de durao
     */
    
    public int getPlayingTime() {
      return this.playingTime;
    }

    /**
     * Devolve uma String com a representaao do objecto ItemMultimedia
     * 
     * @return String com os campos existentes
     */
    
      public String toString() {
        StringBuffer sres = new StringBuffer();
      
        sres.append("ItemMultimedia {");
        sres.append("Title: "); sres.append(this.title);
        sres.append("Playing Time: "); sres.append(this.playingTime);
        sres.append("Got It: "); sres.append(this.gotIt);
        sres.append("Comment: "); sres.append(this.comment);
        sres.append("}"); 
  
        return sres.toString();
    }
    
    
    /*
     * Mtodo que devolve um booleano indicando se dois ItemMultimedia tem os mesmos 
     * valores para as variveis de instncia
     * 
     * @return boolean indicando se so iguais ou no
     */
    
    public boolean equals(Object o) {
      
      if(this == o)
        return true;
      /* Verificar se o objecto passado no  null e se os dois objectos so MESMO da mesma classe.
       * Caso contrrio no poderiam ser comparados
       */  
      
      if ((o != null) && this.getClass() == o.getClass())  {
          ItemMultimedia anObj = (ItemMultimedia)o;
          return(this.title.equals(anObj.getTitle()) && this.playingTime == anObj.getPlayingTime() &&
                 this.gotIt == anObj.getOwn() && this.comment.equals(anObj.getComment()));
       }
       else 
         return false;
    }
    
    /*
     * O mtodo clone  abstract em ItemMultimedia, uma vez que no  possvel 
     * criar instncias de ItemMultimedia.
     * Est declarado como abstract para as subclasses o implementarem.
     */
    
    public abstract Object clone();
    
}
